
/*! 
    \file vhash.h
    \brief A documented header file containing the vhash 
*/


#ifndef VHASH_H
#define VHASH_H

#include <stdlib.h>

#include "vlist.h"

#ifdef _WIN32_WCE
#define VHASH_CDECL __cdecl
#else
#define VHASH_CDECL 
#endif


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*!
    The vhash is a linear probing hash.

    This vhash assumes all keys are unique.
    If not remove the old key/item first the add the new one.

    The vhash can operate on void pointers or string keys.
    Do not mix these uses.

    This auto resizes at 50% capacity for
    best perfomance choose a size 2*max items expected.
*/

typedef void *(VHASH_CDECL *vmalloc_t) (size_t);
typedef void (VHASH_CDECL *vfree_t) (void*);
typedef struct vhash_s {
    struct hash_node_s * table;
    unsigned long count;
    unsigned long table_size;
    long key_length;
    char * key_string;
#if 0
    void *(VHASH_CDECL *malloc) (size_t);
    void (VHASH_CDECL *free) (void*);
#else
	vmalloc_t malloc;
	vfree_t	free;

#endif
} vhash_t;

/*! \def VHASH_STATUS_FAILED
    \brief A failure status
*/
#define VHASH_STATUS_FAILED 0
/*! \def VHASH_STATUS_SUCCESS
    \brief A success status
*/
#define VHASH_STATUS_SUCCESS 1
/*! \def VHASH_STATUS_INSERTED
    \brief A item was inserted 
*/
#define VHASH_STATUS_INSERTED 2

/*! \def vhash_status_t
    \brief A type for function status
*/
typedef int vhash_status_t;

/*!
    \brief This function creates and initalises a vhash structure.
    \param table_size How big to create the hash (greater than 0 please)
    \param vmalloc A malloc function for vhash to use    
    \param vfree A free function for vhash to use    
    \return The vhash structure 
*/
_W3DTK_API vhash_t * new_vhash(
    unsigned long table_size,
	vmalloc_t vmalloc,
	vfree_t	vfree);
	
#if 0
    void *(VHASH_CDECL *vhash_malloc) (size_t),
    void (VHASH_CDECL *vhash_free) (void *));
#endif

/*!
    \brief This function destroys and cleans up a vhash structure.
    \param vhash The vhash to delete
*/
_W3DTK_API void delete_vhash(vhash_t * vhash);


_W3DTK_API void vhash_flush(vhash_t * vhash);


_W3DTK_API vmalloc_t vhash_malloc(vhash_t * vhash);
_W3DTK_API vfree_t vhash_free(vhash_t * vhash);
/*!
    \brief This function rebuilds the hash a specifed size never rebuild less than 2*count
    \param vhash The vhash to operate on
    \param table_size The new size
    \return a vhash_status_t the result of the rebuild
*/
_W3DTK_API vhash_status_t vhash_rebuild_table(
    vhash_t * vhash,
    unsigned long table_size);

/*!
    \brief This function returns the number of items in the vhash.
    \param vhash The vhash to operate on
    \return The number of items in the vhash
*/
_W3DTK_API unsigned long vhash_count(vhash_t* vhash);

/*!
    \brief This function calls a function once per item in the vhash.
    \brief Your function returns VHASH_STATUS_SUCCESS to keep mapping.
    \brief Your function returns VHASH_STATUS_FAILED to stop mapping.
    \param v The vhash to operate on
    \param function The function to call (item,key,user_data) 
    \param user_data A void pointer to pass with each item
*/
_W3DTK_API void vhash_map_function(
    vhash_t * v,
    void(*function)(void*, void*, void*),
    void * user_data);

/*! \def vhash_pair_t
    \brief A type for key item pairs
*/
_W3DTK_API typedef struct vhash_pair_s {
    void * key;
    void * item;
} vhash_pair_t;

/*! \def vhash_string_key_pair_t
    \brief A type for key item pairs when the vhash is storing strings
*/
_W3DTK_API typedef struct vhash_string_key_pair_s {
    const char * key;
    void * item;
} vhash_string_key_pair_t;

/*!
    \brief This function builds a vlist of item key pairs.
    \param vhash The vhash to operate on
    \param vlist A vlist to populate with key item pairs
    \param vhash_pair_malloc A malloc function to user to allocate the key item pairs
    \return a vhash_status_t the result of the action
*/
_W3DTK_API vhash_status_t vhash_to_vlist(
    vhash_t * vhash,
    vlist_t * vlist,
    void *(VHASH_CDECL *vhash_pair_malloc) (size_t));

_W3DTK_API int vhash_item_set_to_vlist(
	vhash_t * vhash,
	vlist_t * vlist,
	void * in_key);

/*!
    \brief This function replaces or adds a item to the vhash.
    \param v The vhash to operate on
    \param in_key A item key
    \param new_item The item to add to the vhash
    \param replaced_item A pointer to a void pointer that a replaced item can be write to
    \return a vhash_status_t the result of the action (returns VHASH_STATUS_SUCCESS always)
*/
_W3DTK_API vhash_status_t vhash_replace_item(
    vhash_t * v,
    void * in_key, 
    void * new_item,
    void ** replaced_item);

/*!
    \brief This function adds a item to the vhash.
    \param v The vhash to operate on
    \param in_key A item key
    \param item The item to add to the vhash
    \return a vhash_status_t the result of the action (returns VHASH_STATUS_SUCCESS always)
*/
_W3DTK_API vhash_status_t vhash_insert_item(
    vhash_t * v,
    void * in_key,
    void * item);

/*!
    \brief This function removes an item from the vhash.
    \param v The vhash to operate on
    \param in_key A item key
    \param removed_item A pointer to a void pointer that a removed item can be write to
    \return a vhash_status_t the result of the action 
*/
_W3DTK_API vhash_status_t vhash_remove_item(
    vhash_t * v,
    void * in_key,
    void ** removed_item);

/*!
    \brief This function looks up an item from the vhash.
    \param v The vhash to operate on
    \param in_key A item key
    \param out_item A pointer to a void pointer that a looked up item can be write to
    \return a vhash_status_t the result of the action
*/
_W3DTK_API vhash_status_t vhash_lookup_item(
    const vhash_t * v,
    void * in_key,
    void ** out_item);

/*!
    \brief This function behaves exactly like vhash_lookup_item, except it skips a number of 
        matches equal to the given 'n'.  When hash keys are not guaranteed to be unique, this function
        allows access to all of the items that match a given key.
    \param v The vhash to operate on
    \param in_key A item key
    \param n The number of matches to skip
    \param out_item A pointer to a void pointer that a looked up item can be write to
    \return a vhash_status_t the result of the action
*/
_W3DTK_API vhash_status_t vhash_lookup_nth_item(
    const vhash_t * v,
    void * in_key,
    int n,
    void ** out_item);

/*!
    \brief This function calls a function once per item in the vhash.
    \brief The keys are strings.
    \brief Your function returns VHASH_STATUS_SUCCESS to keep mapping.
    \brief Your function returns VHASH_STATUS_FAILED to stop mapping.
    \param v The vhash to operate on
    \param function The function to call (item,key,user_data) 
    \param user_data A void pointer to pass with each item
*/
_W3DTK_API void vhash_string_key_map_function(
    vhash_t * v,
    void(*function)(void*, const char *, void*),
    void * user_data);

/*!
    \brief This function replaces or adds a item to the vhash.
    \brief The keys are strings.
    \brief replaced_item is valid only if return status is VHASH_STATUS_SUCCESS 
    \brief returns VHASH_STATUS_INSERTED is no item with same key existed
    \param v The vhash to operate on
    \param string_key A null terminated item key
    \param new_item The item to add to the vhash
    \param replaced_item A pointer to a void pointer that a replaced item can be write to
    \return a vhash_status_t the result of the action
*/
_W3DTK_API vhash_status_t vhash_replace_string_key_item(
    vhash_t * v,
    const char * string_key, 
    void * new_item,
    void ** replaced_item);

/*!
    \brief This function adds a item to the vhash.
    \brief The keys are strings.
    \param v The vhash to operate on
    \param string_key A null terminated item key
    \param item The item to add to the vhash
    \return a vhash_status_t the result of the action
*/
_W3DTK_API vhash_status_t vhash_insert_string_key_item(
    vhash_t * v,
    const char * string_key, 
    void * item);

/*!
    \brief This function removes an item from the vhash.
    \brief The keys are strings.
    \param v The vhash to operate on
    \param string_key A null terminated item key
    \param removed_item A pointer to a void pointer that a removed item can be write to
    \return a vhash_status_t the result of the action
*/
_W3DTK_API vhash_status_t vhash_remove_string_key_item(
    vhash_t * v,
    const char * string_key,
    void ** removed_item);

/*!
    \brief This function looks up an item from the vhash.
    \brief The keys are strings.
    \param v The vhash to operate on
    \param string_key A null terminated item key
    \param out_item A pointer to a void pointer that a looked up item can be write to
    \return a vhash_status_t the result of the action
*/
_W3DTK_API vhash_status_t vhash_lookup_string_key_item(
    vhash_t * v,
    const char * string_key,
    void ** out_item);

/*!
    \brief This function behaves exactly like vhash_lookup_string_key_item, except it skips a number of 
        matches equal to the given 'n'.  When hash keys are not guaranteed to be unique, this function
        allows access to all of the items that match a given key.
    \brief The keys are strings.
    \param v The vhash to operate on
    \param string_key A null terminated item key
    \param n The number of matches to skip
    \param out_item A pointer to a void pointer that a looked up item can be write to
    \return a vhash_status_t the result of the action
*/
_W3DTK_API vhash_status_t vhash_lookup_nth_string_key_item(
    vhash_t * v,
    const char * string_key,
    int n,
    void ** out_item);


/*!
    \brief This function builds a string key item pair from the vhash.
    \param vhash The vhash to operate on
    \param vlist A vlist to populate with string key item pairs
    \param (VHASH_CDECL *vhash_pair_malloc) A malloc function to user to allocate the key item pairs
    \return a vhash_status_t the result of the action
*/
_W3DTK_API vhash_status_t vhash_string_keys_to_vlist(
    vhash_t * vhash,
    vlist_t * vlist,
    void *(VHASH_CDECL *vhash_pair_malloc) (size_t));


/*THIS IS OBSOLETE*/
_W3DTK_API void vhash_insert(
    vhash_t * vhash,
    void * key, void * item);

/*THIS IS OBSOLETE*/
_W3DTK_API void * vhash_remove(
    vhash_t * vhash,
    void * key);

/*THIS IS OBSOLETE*/
_W3DTK_API void * vhash_lookup(
    vhash_t * vhash,
    void * key);


/*THIS IS OBSOLETE*/
_W3DTK_API void vhash_insert_string_key(
    vhash_t * v,
    const char * string, 
    void * item);

/*THIS IS OBSOLETE*/
_W3DTK_API void * vhash_remove_string_key(
    vhash_t * v,
    const char * string);

/*THIS IS OBSOLETE*/
_W3DTK_API void * vhash_lookup_string_key(
    vhash_t * v,
    const char * string);

#ifdef __cplusplus
} /* extern "C" */
#endif /* __cplusplus */

/* template wrapper */
#ifdef __cplusplus
extern "C++" {
	
template <typename T1, typename T2> class VHash {

public:

	class Pair {
		public:
			T1 Key() {return (T1)(V_POINTER_SIZED_INT)vpair.key;};
			T2 Item() {return (T2)(V_POINTER_SIZED_INT)vpair.item;};
		private:
			vhash_pair_t vpair;
	};


	typedef VList<Pair*> PairList;

    VHash(int initial_size=16)
        { m_vhash = new_vhash(initial_size,malloc,free); size_assert(); };

    VHash(void * (VHASH_CDECL *  vhash_malloc) (size_t), void (VHASH_CDECL *vhash_free) (void *))
        {m_vhash = new_vhash(16, vhash_malloc,vhash_free); size_assert(); };

    ~VHash() {delete_vhash(m_vhash);};

    void Flush() { vhash_flush(m_vhash); };

    unsigned long Count() { return (vhash_count(m_vhash)); };

	PairList * GetPairList() {

		struct vlist_s *lv = new_vlist(vhash_malloc(m_vhash),vhash_free(m_vhash));

		if (VHASH_STATUS_FAILED == vhash_to_vlist(m_vhash, lv, vhash_malloc(m_vhash))) {
			delete_vlist(lv);
			return 0;
		}
		PairList * pl = new PairList(lv);
		pl->SetFreeItemsOnDelete();
		return pl;
	};
	
	void MapFunction(void(*function)(T1, T2, void const * const), void const * const user_data) {
		MapFunctionData data(function,user_data);
		vhash_map_function(m_vhash, &map_function_helper, (void*)&data);
	};

    vhash_status_t ReplaceItem(T1 key, T2 item, T2 *replaced_item){
		void * tmp_item = 0;
		vhash_status_t retval = vhash_replace_item(m_vhash, (void*)(V_POINTER_SIZED_INT)key, (void*)(V_POINTER_SIZED_INT)item, &tmp_item);
		if (VHASH_STATUS_SUCCESS == retval)
			*replaced_item = (T2)(V_POINTER_SIZED_INT)tmp_item;
		return retval;
	};

    vhash_status_t InsertItem(T1 key, T2 item){
		return vhash_insert_item(m_vhash, (void*)(V_POINTER_SIZED_INT)key, (void*)(V_POINTER_SIZED_INT)item);
	};

    vhash_status_t LookupItem(T1 key, T2 *looked_up_item){
		void * tmp_item = 0;
		vhash_status_t retval = vhash_lookup_item(m_vhash, (void*)(V_POINTER_SIZED_INT)key, &tmp_item);
		if (VHASH_STATUS_SUCCESS == retval)
			*looked_up_item = (T2)(V_POINTER_SIZED_INT)tmp_item;
		return retval;
	};

    T2 Lookup(T1 key)
        {return (T2)(V_POINTER_SIZED_INT)vhash_lookup(m_vhash, (void*)(V_POINTER_SIZED_INT)key); };
	
	
    vhash_status_t RemoveItem(T1 key, T2 *removed_item){
		void * tmp_item = 0;
		vhash_status_t retval = vhash_remove_item(m_vhash, (void*)(V_POINTER_SIZED_INT)key, &tmp_item);
		if (VHASH_STATUS_SUCCESS == retval)
			*removed_item = (T2)(V_POINTER_SIZED_INT)tmp_item;
		return retval;
	};
		

	vhash_status_t LookupNthItem(T1 key,int n, T2 *out_item){
		void * tmp_item = 0;
		vhash_status_t retval = vhash_lookup_nth_item(m_vhash, (void*)(V_POINTER_SIZED_INT)key, n, &tmp_item);
		if (VHASH_STATUS_SUCCESS == retval)
			*out_item = (T2)(V_POINTER_SIZED_INT)tmp_item;
		return retval;
	};

private:
    struct vhash_s *m_vhash;

	typedef  void(*MapFunctionType)(T1, T2, void const * const);
	
	class MapFunctionData {
		public:	
		MapFunctionData(MapFunctionType function=0, void const * const data=0) {
			m_function=function;
			m_data=(void*)data;
		};
		MapFunctionType m_function;
		void * m_data;
	};

	static void map_function_helper(void* in_a, void* in_b, void*in_data) {
		MapFunctionData *data = (MapFunctionData *)in_data;
		T1 a = (T1)(V_POINTER_SIZED_INT)in_a;
		T2 b = (T2)(V_POINTER_SIZED_INT)in_b;
		data->m_function(a,b,data->m_data);
	};

    /* VHash cannot use any type that cannot be converted to and from a (void *) */
    void size_assert() {
        VLIST_COMPILE_TIME_ASSERT(sizeof(T1) <= sizeof(void *));
        VLIST_COMPILE_TIME_ASSERT(sizeof(T2) <= sizeof(void *));
    };
};

template <typename T1> class VStringHash {
public:

	class Pair {
		public:
			const char * Key() {return vpair.key;};
			T1 Item() {return (T1)(V_POINTER_SIZED_INT)vpair.item;};
		private:
			vhash_string_key_pair_t vpair;
	};


	typedef VList<Pair*> PairList;

    VStringHash(int initial_size=16)
        { m_vhash = new_vhash(initial_size,malloc,free); size_assert(); };

    VStringHash(void * (VHASH_CDECL *  vhash_malloc) (size_t), void (VHASH_CDECL *vhash_free) (void *))
        {m_vhash = new_vhash(16, vhash_malloc,vhash_free); size_assert(); };

    ~VStringHash() {delete_vhash(m_vhash);};

    void Flush() { vhash_flush(m_vhash); };

    unsigned long Count() { return (vhash_count(m_vhash)); };
	
    vhash_status_t ReplaceItem(const char *key, T1 item, T1 *replaced_item){
		void * tmp_item = 0;
		vhash_status_t retval = vhash_replace_string_key_item(m_vhash, (const char *)key, (void*)(V_POINTER_SIZED_INT)item, &tmp_item);
		if (VHASH_STATUS_SUCCESS == retval)
			*replaced_item = (T1)(V_POINTER_SIZED_INT)tmp_item;
		return retval;
	};

    vhash_status_t InsertItem(const char *key, T1 item)
        {return vhash_insert_string_key_item(m_vhash, (const char *)key, (void*)(V_POINTER_SIZED_INT)item); };
	
    vhash_status_t LookupItem(const char *key, T1 *looked_up_item){
		void * tmp_item = 0;
		vhash_status_t retval = vhash_lookup_string_key_item(m_vhash, (const char*)key, &tmp_item);
		if (VHASH_STATUS_SUCCESS == retval)
			*looked_up_item = (T1)(V_POINTER_SIZED_INT)tmp_item;
		return retval;	};

    T1 Lookup(const char *key)
		{return (T1)(V_POINTER_SIZED_INT)vhash_lookup_string_key(m_vhash, (const char*)key); };
	
	
    vhash_status_t RemoveItem(const char *key, T1 *removed_item){
		void * tmp_item = 0;
		vhash_status_t retval = vhash_remove_string_key_item(m_vhash, (const char*)key, &tmp_item);
		if (VHASH_STATUS_SUCCESS == retval)
			*removed_item = (T1)(V_POINTER_SIZED_INT)tmp_item;
		return retval;
	};

	
	vhash_status_t LookupItem(const char *key, int n, T1 *out_item){
		void * tmp_item = 0;
		vhash_status_t retval = vhash_lookup_nth_string_key_item(m_vhash, (const char*)key, n, &tmp_item);
		if (VHASH_STATUS_SUCCESS == retval)
			*out_item = (T1)(V_POINTER_SIZED_INT)tmp_item;
		return retval;
	};

	PairList * GetPairList() {

		struct vlist_s *lv = new_vlist(vhash_malloc(m_vhash),vhash_free(m_vhash));

		if (VHASH_STATUS_FAILED == vhash_string_keys_to_vlist(m_vhash, lv, vhash_malloc(m_vhash))) {
			delete_vlist(lv);
			return 0;
		}
		PairList * pl = new PairList(lv);
		pl->SetFreeItemsOnDelete();
		return pl;
	};
	
	void MapFunction(void(*function)(T1, const char *, void const * const), void const * const user_data) {
		MapFunctionData data(function,user_data);
		vhash_string_key_map_function(m_vhash, &map_function_helper, (void*)&data);
	};

private:
    struct vhash_s *m_vhash;

	typedef  void(*MapFunctionType)(T1, const char *, void const * const);
	
	class MapFunctionData {
		public:	
		MapFunctionData(MapFunctionType function=0, void const * const data=0) {
			m_function=function;
			m_data=(void*)data;
		};
		MapFunctionType m_function;
		void * m_data;
	};

	static void map_function_helper(void* in_a, const char * in_b, void*in_data) {
		MapFunctionData *data = (MapFunctionData *)in_data;
		T1 a = (T1)(V_POINTER_SIZED_INT)in_a;
		data->m_function(a,in_b,data->m_data);
	};

    /* VHash cannot use any type that cannot be converted to and from a (void *) */
    void size_assert() {
        VLIST_COMPILE_TIME_ASSERT(sizeof(T1) <= sizeof(void *));
    };
};
} /* extern "C++" */
#endif
#endif /*VHASH_H*/


